# Generated by roxygen2: do not edit by hand

export(add_root_info)
export(bind_tip)
export(bind_tip_df)
export(get_graft_status)
export(get_one_tree)
export(get_tree)
export(rm_stars)
export(sp_list_df)
import(data.table)
importFrom(Rcpp,evalCpp)
importFrom(fastmatch,"%fin%")
importFrom(stats,setNames)
useDynLib(rtrees, .registration = TRUE)
