#' Fetching phylogenies from mega-trees
#'
#' @name rtrees-package
#' @aliases rtrees
#' @docType package
#' @author Daijiang Li <daijianglee@@gmail.com>
#' @keywords internal
#' @useDynLib rtrees, .registration = TRUE
#' @importFrom Rcpp evalCpp
"_PACKAGE"
