# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

bind_tip_core_cpp <- function(parent, node, branch_length, label, is_tip, where, tip_label, node_label_str, frac, new_node_above, node_height) {
    .Call(`_rtrees_bind_tip_core_cpp`, parent, node, branch_length, label, is_tip, where, tip_label, node_label_str, frac, new_node_above, node_height)
}

graft_all_cpp <- function(parent0, node0, bl0, label0, is_tip0, where_labels, new_tip_labels, new_node_labels, fracs, above_flags, node_heights_vec) {
    .Call(`_rtrees_graft_all_cpp`, parent0, node0, bl0, label0, is_tip0, where_labels, new_tip_labels, new_node_labels, fracs, above_flags, node_heights_vec)
}

fast_internal_offspring_cpp <- function(parent, node, is_tip, root_node) {
    .Call(`_rtrees_fast_internal_offspring_cpp`, parent, node, is_tip, root_node)
}

