% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{\%fin\%}
\alias{\%fin\%}
\title{Faster match of vectors}
\description{
See \code{fastmatch::\link[fastmatch:fmatch]{\%fin\%}} for details.
}
\keyword{internal}
